package com.gxzc.zen.orm.config

import com.alibaba.druid.pool.DruidDataSource
import com.baomidou.mybatisplus.spring.MybatisSqlSessionFactoryBean
import com.baomidou.mybatisplus.spring.boot.starter.MybatisPlusProperties
import com.baomidou.mybatisplus.spring.boot.starter.SpringBootVFS
import com.gxzc.zen.orm.DynamicMultipleDataSource
import com.gxzc.zen.orm.contants.DSKey
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Primary
import org.springframework.jdbc.datasource.DataSourceTransactionManager
import org.springframework.transaction.PlatformTransactionManager
import org.springframework.transaction.annotation.TransactionManagementConfigurer
import javax.sql.DataSource

/**
 *
 * @author NorthLan at 2018/1/29
 */
@Configuration
@EnableConfigurationProperties(MybatisPlusProperties::class)
class MultipleDataSourceConfig : TransactionManagementConfigurer{

    @Autowired
    private lateinit var properties: MybatisPlusProperties

    @Bean(DSKey.DSKEY_SYS)
    @ConfigurationProperties(prefix = "datasource.sys")
    @Primary
    fun dataSourceSys(): DataSource {
        return DruidDataSource()
    }

    @Bean(DSKey.DSKEY_BUSINESS)
    @ConfigurationProperties(prefix = "datasource.bus")
    fun dataSourceBus(): DataSource {
        return DruidDataSource()
    }

    @Bean
    fun multipleDataSource(): DynamicMultipleDataSource {
        val dataSourceSys = dataSourceSys()
        val dataSourceBus = dataSourceBus()
        return DynamicMultipleDataSource().also {
            it.setTargetDataSources(mapOf(
                    Pair(DSKey.DSKEY_SYS, dataSourceSys),
                    Pair(DSKey.DSKEY_BUSINESS, dataSourceBus)))
            it.setDefaultTargetDataSource(dataSourceSys)
        }
    }

    @Bean
    fun transactionManager(): PlatformTransactionManager {
        return DataSourceTransactionManager(multipleDataSource())
    }

    @Bean
    @Primary
    fun mybatisSqlSessionFactoryBean(dynamicMultipleDataSource: DynamicMultipleDataSource): MybatisSqlSessionFactoryBean {
        return MybatisSqlSessionFactoryBean().also {
            it.setDataSource(dynamicMultipleDataSource)
            it.vfs = SpringBootVFS::class.java
            it.setConfiguration(properties.configuration)
            it.setGlobalConfig(properties.globalConfig?.convertGlobalConfiguration())
            it.setMapperLocations(properties.resolveMapperLocations())
        }
    }

    override fun annotationDrivenTransactionManager(): PlatformTransactionManager {
        return transactionManager()
    }

//    @Bean
//    fun sqlSessionFactory(dynamicMultipleDataSource: DynamicMultipleDataSource): SqlSessionFactory {
//        return SqlSessionFactoryBean().let {
//            it.setDataSource(dynamicMultipleDataSource)
//            it.`object`
//        }
//    }
}